@@ -898,6 +898,20 @@ key_is_laptop (MetaOutputKey *key)
     g_str_has_prefix (key->connector, "eDP");
 }
 
+static gboolean
+output_is_laptop (MetaOutput *output)
+{
+  /* FIXME: extend with better heuristics */
+  switch (output->connector_type)
+    {
+    case META_CONNECTOR_TYPE_eDP:
+    case META_CONNECTOR_TYPE_LVDS:
+      return TRUE;
+    default:
+      return FALSE;
+    }
+}
+
 static gboolean
 laptop_display_is_on (MetaConfiguration *config)
 {
@@ -956,8 +970,7 @@ make_laptop_lid_config (MetaConfiguration  *reference)
       new->keys[i].product = g_strdup (current_key->product);
       new->keys[i].serial = g_strdup (current_key->serial);
 
-      if (g_str_has_prefix (current_key->connector, "LVDS") ||
-          g_str_has_prefix (current_key->connector, "eDP"))
+      if (key_is_laptop (current_key))
         {
           new->outputs[i].enabled = FALSE;
           x_after = current_output->rect.x;
@@ -1061,8 +1074,7 @@ find_primary_output (MetaOutput *outputs,
 
   for (i = 0; i < n_outputs; i++)
     {
-      if (g_str_has_prefix (outputs[i].name, "LVDS") ||
-          g_str_has_prefix (outputs[i].name, "eDP"))
+      if (output_is_laptop (&outputs[i]))
         return &outputs[i];
     }
 
